Index: apps/app_system.c
===================================================================
--- apps/app_system.c	(revision 89522 (1.6.0.3) (was 168546))
+++ apps/app_system.c	(working copy)
@@ -33,7 +33,11 @@
 #include "asterisk/module.h"
 #include "asterisk/app.h"
 #include "asterisk/channel.h"	/* autoservice */
+#include "asterisk/strings.h"
+#include "asterisk/threadstorage.h"
 
+AST_THREADSTORAGE(buf_buf);
+
 static char *app = "System";
 
 static char *app2 = "TrySystem";
@@ -62,6 +66,7 @@
 static int system_exec_helper(struct ast_channel *chan, void *data, int failmode)
 {
 	int res = 0;
+	struct ast_str *buf = ast_str_thread_get(&buf_buf, 16);
 	
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "System requires an argument(command)\n");
@@ -72,7 +77,9 @@
 	ast_autoservice_start(chan);
 
 	/* Do our thing here */
-	res = ast_safe_system((char *)data);
+	ast_str_get_encoded_str(&buf, 0, (char *) data);
+	res = ast_safe_system(ast_str_buffer(buf));
+
 	if ((res < 0) && (errno != ECHILD)) {
 		ast_log(LOG_WARNING, "Unable to execute '%s'\n", (char *)data);
 		pbx_builtin_setvar_helper(chan, chanvar, "FAILURE");
Index: include/asterisk/strings.h
===================================================================
--- include/asterisk/strings.h	(revision (1.6.0.3) (not patched in trunk))
+++ include/asterisk/strings.h	(working copy)
@@ -393,6 +393,39 @@
 }
 )
 
+#define __AST_STR_USED used
+#define __AST_STR_LEN len
+#define __AST_STR_STR str
+/*!\brief Returns the string buffer within the ast_str buf.
+ * \param A pointer to the ast_str string.
+ */
+AST_INLINE_API(
+char * attribute_pure ast_str_buffer(struct ast_str *buf),
+{
+	return buf->__AST_STR_STR;
+}
+)
+
+/*!\brief Returns the current maximum length (without reallocation) of the current buffer.
+ * \param A pointer to the ast_str string.
+ */
+AST_INLINE_API(
+size_t attribute_pure ast_str_size(struct ast_str *buf),
+{
+	return buf->__AST_STR_LEN;
+}
+)
+
+/*! \brief Update the length of the buffer, after using ast_str merely as a buffer.
+ *  \param buf A pointer to the ast_str string.
+ */
+AST_INLINE_API(
+void ast_str_update(struct ast_str *buf),
+{
+	buf->__AST_STR_USED = strlen(buf->__AST_STR_STR);
+}
+)
+
 /*
  * AST_INLINE_API() is a macro that takes a block of code as an argument.
  * Using preprocessor #directives in the argument is not supported by all
Index: include/asterisk/app.h
===================================================================
--- include/asterisk/app.h	(revision (1.6.0.3) (was 168546))
+++ include/asterisk/app.h	(working copy)
@@ -480,6 +480,12 @@
 /*! \brief Decode an encoded control or extended ASCII character */
 int ast_get_encoded_char(const char *stream, char *result, size_t *consumed);

+/*! \brief Decode a stream of encoded control or extended ASCII characters */
+int ast_get_encoded_str(const char *stream, char *result, size_t result_len);
+
+/*! \brief Decode a stream of encoded control or extended ASCII characters */
+int ast_str_get_encoded_str(struct ast_str **str, int maxlen, const char *stream);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
Index: main/app.c
===================================================================
--- main/app.c	(revision 142749 (1.6.0.3) (was 168546))
+++ main/app.c	(working copy)
@@ -1767,6 +1767,50 @@
 		*result = *stream;
 		*consumed = 1;
 	}
 	return 0;
+}
+
+int ast_get_encoded_str(const char *stream, char *result, size_t result_size)
+{
+        char *cur = result;
+        size_t consumed;
+
+	if (strchr(stream, '\\')) {
+		while (cur < result + result_size - 1 && !ast_get_encoded_char(stream, cur, &consumed)) {
+			cur++;
+			stream += consumed;
+		}
+		*cur = '\0';
+	} else {
+		ast_copy_string(result, stream, result_size);
+        }
+        return 0;
+}
+ 
+int ast_str_get_encoded_str(struct ast_str **str, int maxlen, const char *stream)
+{
+	char next, *buf;
+	size_t offset = 0;
+	size_t consumed;
+
+	if (strchr(stream, '\\')) {
+		while (!ast_get_encoded_char(stream, &next, &consumed)) {
+			if (offset + 2 > ast_str_size(*str) && maxlen > -1) {
+				ast_str_make_space(str, maxlen > 0 ? maxlen : ast_str_size(*str) * 2);
+			}
+			if (offset + 2 > ast_str_size(*str)) {
+				break;
+			}
+			buf = ast_str_buffer(*str);
+			buf[offset++] = next;
+			stream += consumed;
+		}
+		buf = ast_str_buffer(*str);
+		buf[offset++] = '\0';
+		ast_str_update(*str);
+	} else {
+		ast_str_set(str, maxlen, "%s", stream);
+	}
+	return 0;
 }
 
Index: main/pbx.c
===================================================================
--- main/pbx.c	(revision 160389 (1.6.0.3) (was 168546))
+++ main/pbx.c	(working copy)
@@ -124,6 +124,7 @@
 struct ast_app;
 
 AST_THREADSTORAGE(switch_data);
+AST_THREADSTORAGE(setvar_buf);
 
 /*!
    \brief ast_exten: An extension
@@ -8063,6 +8064,7 @@
 int pbx_builtin_setvar(struct ast_channel *chan, void *data)
 {
 	char *name, *value, *mydata;
+	struct ast_str *buf = ast_str_thread_get(&setvar_buf, 16);
 
 	if (ast_compat_app_set) {
 		return pbx_builtin_setvar_multiple(chan, data);
@@ -8076,10 +8078,11 @@
 	mydata = ast_strdupa(data);
 	name = strsep(&mydata, "=");
 	value = mydata;
+	ast_str_get_encoded_str(&buf, 0, value);
 	if (strchr(name, ' '))
 		ast_log(LOG_WARNING, "Please avoid unnecessary spaces on variables as it may lead to unexpected results ('%s' set to '%s').\n", name, mydata);
 
-	pbx_builtin_setvar_helper(chan, name, value);
+	pbx_builtin_setvar_helper(chan, name, ast_str_buffer(buf));
 	return(0);
 }
 
@@ -8087,6 +8090,7 @@
 {
 	char *data;
 	int x;
+	struct ast_str *buf = ast_str_thread_get(&setvar_buf, 16);
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(pair)[24];
 	);
@@ -8106,7 +8111,8 @@
 	for (x = 0; x < args.argc; x++) {
 		AST_NONSTANDARD_APP_ARGS(pair, args.pair[x], '=');
 		if (pair.argc == 2) {
-			pbx_builtin_setvar_helper(chan, pair.name, pair.value);
+			ast_str_get_encoded_str(&buf, 0, pair.value);
+			pbx_builtin_setvar_helper(chan, pair.name, ast_str_buffer(buf));
 			if (strchr(pair.name, ' '))
 				ast_log(LOG_WARNING, "Please avoid unnecessary spaces on variables as it may lead to unexpected results ('%s' set to '%s').\n", pair.name, pair.value);
 		} else if (!chan) {
